@@ -8,9 +8,16 @@
 ###########################################################################
 
 LML <- function(Model=NULL, Data=NULL, Modes=NULL, theta=NULL, LL=NULL,
-     method="NSIS")
+     Covar=NULL, method="NSIS")
      {
      LML.out <- list(LML=NA, VarCov=NA)
+     if(!is.null(Modes) & !is.null(theta) & !is.null(LL) &
+          !is.null(Covar) & method == "GD") {
+          log.g.theta <- dmvn(theta, Modes, Covar, log=TRUE)
+          LML <- log(1 / mean(exp(log.g.theta - LL)))
+          ### Output
+          LML.out <- list(LML=LML, VarCov=NA)
+          }
      if(!is.null(LL) & method == "HME") {
           med <- median(LL)
           LML <- med - log(mean(exp(-LL + med)))
@@ -18,89 +25,34 @@ LML <- function(Model=NULL, Data=NULL, Modes=NULL, theta=NULL, LL=NULL,
           LML.out <- list(LML=LML, VarCov=NA)
           }
      if(!is.null(Model) & !is.null(Data) & !is.null(Modes) &
-          (method == "LME1")) {
-          Interval <- 1.0E-6
+          (method == "LME")) {
+          Interval <- 1.0e-6
           parm.len <- length(Modes)
-          eps <- Interval * Modes
-          timetest <- system.time(Model(Modes, Data))
-          timeest <- parm.len^2 * 2 * as.vector(timetest[3]) * 1.1 / 60
-          cat("\nApproximating LML should take just over",
-               round(timeest), "minutes.\n")
-          Approx.Hessian <- Hessian(Model, Modes, Data)
-          Inverse.test <- try(-as.inverse(Approx.Hessian), silent=TRUE)
-          if(!inherits(Inverse.test, "try-error")) {
-               VarCov <- Inverse.test
-               diag(VarCov) <- ifelse(diag(VarCov) <= 0,
-                    .Machine$double.eps, diag(VarCov))}
+          if(is.null(Covar)) {
+               eps <- Interval * Modes
+               Approx.Hessian <- Hessian(Model, Modes, Data)
+               VarCov <- try(-as.inverse(Approx.Hessian), silent=TRUE)
+               if(!inherits(VarCov, "try-error"))
+                    diag(VarCov)[which(diag(VarCov) <= 0)] <- .Machine$double.eps
+               else {
+                    cat("\nWARNING: Failure to solve matrix inversion of ",
+                         "Approx. Hessian in LML.\n", sep="")
+                    cat("NOTE: Identity matrix is supplied instead.\n")
+                    VarCov <- diag(parm.len)}
+               }
           else {
-               cat("\nWARNING: Failure to solve matrix inversion of ",
-                    "Approx. Hessian in LML.\n", sep="")
-               cat("NOTE: Identity matrix is supplied instead.\n")
-               VarCov <- diag(parm.len)}
+               VarCov <- Covar
+               rm(Covar)}
           ### Logarithm of the Marginal Likelihood
           LML <- NA
           options(warn=-1)
-          LML.test <- try(parm.len/2 * log(2*pi) + 0.5*log(det(VarCov)) +
-               as.vector(Model(Modes, Data)[[1]]), silent=TRUE)
+          LML.test <- try(parm.len/2 * log(2*pi) + 0.5*logdet(VarCov) +
+               as.vector(Model(Modes, Data)[["LP"]]), silent=TRUE)
           if(!inherits(LML.test, "try-error")) LML <- LML.test[1]
           options(warn=0)
           ### Output
           LML.out <- list(LML=LML, VarCov=VarCov)
           }
-     if(!is.null(Model) & !is.null(Data) & !is.null(Modes) &
-          (method == "LME2")) {
-          parm.len <- length(Modes)
-          KoschalHess <- function(pars, fun, ...,
-               .relStep=(.Machine$double.eps)^(1/3), minAbsPar=0)
-               {
-               pars <- as.numeric(pars)
-               npar <- length(pars)
-               incr <- ifelse(abs(pars) <= minAbsPar, minAbsPar * .relStep,
-                    abs(pars) * .relStep)
-               baseInd <- diag(npar)
-               frac <- c(1, incr, incr^2)
-               cols <- list(0, baseInd, -baseInd)
-               for (i in seq_along(pars)[-npar]) {
-                    cols <- c(cols, list(baseInd[, i] + baseInd[, -(1:i)]))
-                    frac <- c(frac, incr[i] * incr[-(1:i)])
-                    }
-               indMat <- do.call("cbind", cols)
-               shifted <- pars + incr * indMat
-               indMat <- t(indMat)
-               Xcols <- list(1, indMat, indMat^2)
-               for (i in seq_along(pars)[-npar])
-                    Xcols <- c(Xcols, list(indMat[, i] * indMat[, -(1:i)]))
-               coefs <- solve(do.call("cbind", Xcols) ,
-                    apply(shifted, 2, fun, ...) )/frac
-               Hess <- diag(coefs[1 + npar + seq_along(pars)], ncol=npar)
-               Hess[row(Hess) > col(Hess)] <- coefs[-(1:(1 + 2 * npar))]
-               list(mean=coefs[1], gradient=coefs[1 + seq_along(pars)],
-                    Hessian=(Hess + t(Hess)))
-               }
-          Koschal.test <- try(KoschalHess(Modes,
-               function(x) Model(x, Data)[[1]])$Hessian, silent=TRUE)
-          if(!inherits(Koschal.test, "try-error"))
-               Approx.Hessian <- Koschal.test
-          Inverse.test <- try(-as.inverse(Approx.Hessian), silent=TRUE)
-          if(!inherits(Inverse.test, "try-error")) {
-               VarCov <- Inverse.test
-               diag(VarCov) <- ifelse(diag(VarCov) <= 0,
-                    .Machine$double.eps, diag(VarCov))}
-          else {
-               cat("\nWARNING: Failure to solve matrix inversion of ",
-                    "Approx. Hessian in LML.\n", sep="")
-               cat("NOTE: Identity matrix is supplied instead.\n")
-               VarCov <- diag(parm.len)}
-          ### Logarithm of the Marginal Likelihood
-          LML <- NA
-          options(warn=-1)
-          LML.test <- try(parm.len/2 * log(2*pi) + 0.5*log(det(VarCov)) +
-               as.vector(Model(Modes, Data)[[1]]), silent=TRUE)
-          if(!inherits(LML.test, "try-error")) LML <- LML.test[1]
-          options(warn=0)
-          ### Output
-          LML.out <- list(LML=LML, VarCov=VarCov)
-     }
      if(!is.null(theta) & !is.null(LL) & (method =="NSIS")) {
           if(!is.matrix(theta)) stop("theta must be a matrix.")
           thetacol <- ncol(theta)
@@ -110,12 +62,12 @@ LML <- function(Model=NULL, Data=NULL, Modes=NULL, theta=NULL, LL=NULL,
                stop("The number of rows in theta differs from the ",
                     "length of LL.")
           if(LLlen < 301) {
-               cat("\nWARNING: At least 301 samples are required for NSIS.\n")
+               cat("\nWARNING: At least 301 samples are required for NSIS in LML.\n")
                return(list(LML=NA, VarCov=NA))}
           if(thetacol > round(LLlen / 2)) {
-               cat("\nWARNING: The number of parameters, ", thetacol,
-                    " exceeds half the number of stationary samples, ",
-                    round(LLlen / 2), " required for NSIS.\n")
+               cat("\nWARNING: The number of parameters,", thetacol,
+                    ",\n exceeds half the number of stationary samples,",
+                    round(LLlen / 2), ",\n required for NSIS.\n")
                return(list(LML=NA, VarCov=NA))}
           cov.prob <- 0.5
           bounds <- matrix(c(-Inf, Inf), 2, ncol(theta))
@@ -262,14 +214,13 @@ LML <- function(Model=NULL, Data=NULL, Modes=NULL, theta=NULL, LL=NULL,
           ml.out <- .MargLL(hist, comps$theta.imp, comps$LL.imp)
           ml.out$samples <- ml.out$samples[is.finite(ml.out$samples)]
           sd.samples <- sd(ml.out$samples) / sqrt(length(ml.out$samples))
-          conf.interval <- qnorm(c(.975, .025)) * sd.samples +
+          conf.interval <- qnorm(c(0.975, 0.025)) * sd.samples +
                mean(ml.out$samples)
-          conf.interval <- ifelse(conf.interval < .Machine$double.eps,
-               .Machine$double.eps, conf.interval)
+          conf.interval[which(conf.interval < .Machine$double.eps)] <- .Machine$double.eps
           conf.interval <- -log(conf.interval)
           options(warn=0)
           LML <- ml.out$mll + hist$norm
-          LML <- ifelse(is.finite(LML), LML, NA)
+          LML[which(!is.finite(LML))] <- NA
           ### Output
           LML.out <- list(LML=LML, VarCov=NA)
           }
